import base64
import json

import boto3
from botocore.exceptions import ClientError
from secretmanager.error_helper import raise_error
from secretmanager.utils import get_app_secret_prefix, get_region, get_logger

logger = get_logger()


class SecretManager:
    aws_service_name = "secretsmanager"
    max_results = 100

    def __init__(self):
        session = boto3.session.Session()
        self.client = session.client(
            service_name=SecretManager.aws_service_name,
            region_name=get_region()
        )
        self.app_secrets_prefix = get_app_secret_prefix()
        logger.info("App secrets prefix is " + self.app_secrets_prefix)

    def get_secrets(self):
        application_secrets = self.filter_app_secrets()
        secret_configs = {}
        try:
            for secret in application_secrets:
                secret_id = secret['Name']
                secret_value = self.get_secret_with_id(secret_id)
                secret_key = secret_id[len(self.app_secrets_prefix) + 1:]
                if secret_key:
                    secret_configs[secret_key] = json.loads(secret_value)
                else:
                    secret_configs[secret_id] = json.loads(secret_value)
                logger.info("Got secret for " + secret_id)
        except ClientError as e:
            raise_error(e)
            return None
        return secret_configs

    def filter_app_secrets(self):
        application_secrets = []
        next_token = None
        while True:
            if next_token:
                secrets = self.client.list_secrets(
                    MaxResults=SecretManager.max_results,
                    NextToken=next_token)
            else:
                secrets = self.client.list_secrets(
                    MaxResults=SecretManager.max_results)

            for secret in secrets['SecretList']:
                if self.app_secrets_prefix in secret['Name']:
                    application_secrets.append(secret)

            next_token = secrets.get('NextToken')
            if not next_token:
                break
        return application_secrets

    def get_secret_with_id(self, secret_id):
        secret_value_response = self.client.get_secret_value(
            SecretId=secret_id
        )
        if 'SecretString' in secret_value_response:
            secret_value = secret_value_response['SecretString']
        else:
            secret_value = base64.b64decode(secret_value_response['SecretBinary'])

        return secret_value



import logging

logger = logging.getLogger()


def raise_error(e):
    if e.response['Error']['Code'] == 'DecryptionFailureException':
        logger.error(" Secrets Manager can't decrypt the protected secret text using the provided KMS key")
        raise e
    elif e.response['Error']['Code'] == 'InternalServiceErrorException':
        logger.error(" An error occurred on the server side.")
        raise e
    elif e.response['Error']['Code'] == 'InvalidParameterException':
        logger.error("You provided an invalid value for a parameter.")
        raise e
    elif e.response['Error']['Code'] == 'InvalidRequestException':
        logger.error(" You provided a parameter value that is not valid for the current state of the resource.")
        raise e
    elif e.response['Error']['Code'] == 'ResourceNotFoundException':
        logger.error(" We can't find the resource that you asked for.")
        raise e



import json
import logging
import os

metadata_endpoint = "http://169.254.169.254/latest/dynamic/instance-identity/document"


def get_region():
    if os.environ.get("APP_ENV") not in ['prod', "preprod"]:
        return None

    # for compatibility with python 2 and 3
    try:
        import urllib.request as url_request
    except ImportError:
        import urllib2 as url_request

    metadata = url_request.urlopen(metadata_endpoint).read()
    metadata_json = json.loads(metadata)
    return metadata_json['region']


def get_app_secret_prefix():
    app_name = os.environ.get("APP_NAME")
    app_env = os.environ.get("APP_ENV")
    return app_name + "_" + app_env


def validate_environment_variables():
    app_env = os.environ.get("APP_ENV")
    app_name = os.environ.get("APP_NAME")

    if not app_name:
        raise Exception("application name is not set")

    if not app_env:
        raise Exception("application env is not set")


def get_logger():
    logger = logging.getLogger("secret-manager")
    logger.setLevel(logging.INFO)
    stream_handler = logging.StreamHandler()
    stream_handler.setLevel(logging.INFO)
    logger.addHandler(stream_handler)
    return logger



import os

from secretmanager.secret_manager import SecretManager
from secretmanager.utils import get_logger, validate_environment_variables

logger = get_logger()
validate_environment_variables()
app_env = os.environ.get("APP_ENV")
secrets = dict
if app_env in ['prod', "preprod", "secret_test"]:
    secret_manager = SecretManager()
    secrets = secret_manager.get_secrets()
else:
    logger.info("Secrets disbled for this environment")



from setuptools import setup, find_packages

setup(name='secret-manager',
      version='0.0.2',
      license='MIT',
      author='Praveen Kumar',
      author_email='praveen.k@blackbuck.com',
      url="https://github.com/BLACKBUCK-LABS/secret-manager-python",
      description='Secret Manager AWS',
      packages=find_packages(exclude=['tests']),
      long_description=open('README.md').read(),
      zip_safe=False)
